﻿// © 2009 Michele Leroux Bustamante. All rights reserved. 
// See http://claimsbasedwpf.codeplex.com for related whitepaper and updates
// For an intro to WCF see Michele's book: Learning WCF, O'Reilly 2007 (updated August 2008 for VS 2008)
// See http://www.thatindigogirl.com for the book code!

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Claims;
using System.Security.Cryptography.X509Certificates;
using WCFClaims = System.IdentityModel.Claims;
using Microsoft.IdentityModel.SecurityTokenService;
using System.IdentityModel.Tokens;
using Microsoft.IdentityModel.Protocols.WSTrust;


namespace RPSTS
{
    public class RPSTS : SecurityTokenService
    {
        public RPSTS(SecurityTokenServiceConfiguration config)
            : base( config )
        {
        }

        protected override IClaimsIdentity GetOutputClaimsIdentity(IClaimsPrincipal principal, RequestSecurityToken request, Scope scope)
        {
            IClaimsIdentity claimsIdentity = new ClaimsIdentity();

            claimsIdentity.Claims.AddRange(CredentialStore.GetClaimsForUser(principal.Identity.Name));

            return claimsIdentity;
        }

        protected override Scope  GetScope(Microsoft.IdentityModel.Claims.IClaimsPrincipal principal, RequestSecurityToken request)
        {

            Scope scope = new Scope(request.AppliesTo.Uri.AbsoluteUri);
            scope.EncryptingCredentials = this.GetCredentialsForAppliesTo(request.AppliesTo);
            scope.SigningCredentials = new X509SigningCredentials(CertificateUtil.GetCertificate(StoreName.My, StoreLocation.LocalMachine, "CN=busta-rpsts.com"));

            return scope;
        }

        private X509EncryptingCredentials GetCredentialsForAppliesTo(EndpointAddress appliesTo)
        {
            if (appliesTo == null || appliesTo.Uri ==null || string.IsNullOrEmpty(appliesTo.Uri.AbsolutePath))
            {
                throw new InvalidRequestException("AppliesTo must be supplied in the RST.");
            }

            X509EncryptingCredentials creds = null;
            if (appliesTo.Uri.AbsoluteUri.StartsWith("http://localhost:8000/TodoListService"))
            {
                creds = new X509EncryptingCredentials(CertificateUtil.GetCertificate(StoreName.TrustedPeople, StoreLocation.LocalMachine, "CN=busta-rp.com"));
            }
            else
                throw new InvalidRequestException(String.Format("Invalid relying party address: {0}", appliesTo.Uri.AbsoluteUri));

            return creds;
        }
    }
}
