﻿// © 2009 Michele Leroux Bustamante. All rights reserved. 
// See http://claimsbasedwpf.codeplex.com for related whitepaper and updates
// For an intro to WCF see Michele's book: Learning WCF, O'Reilly 2007 (updated August 2008 for VS 2008)
// See http://www.thatindigogirl.com for the book code!
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security;
using Microsoft.IdentityModel.Claims;


namespace RPSTS
{
    internal static class CredentialStore
    {

        private static Dictionary<string, string> Credentials { get; set; }
        private static Dictionary<string, List<string>> ClaimsMap {get; set;}
        
        static CredentialStore()
        {
            Credentials = new Dictionary<string, string>();

            Credentials.Add("Admin", "password");
            Credentials.Add("User", "password");
            Credentials.Add("Guest", "password");

            ClaimsMap = new Dictionary<string, List<string>>();

            ClaimsMap.Add("Admin", new List<string> { Constants.Permissions.Create, Constants.Permissions.Read, Constants.Permissions.Update, Constants.Permissions.Delete, Constants.Roles.Administrator, Constants.Roles.User });
            ClaimsMap.Add("User", new List<string> { Constants.Permissions.Create, Constants.Permissions.Read, Constants.Permissions.Update, Constants.Roles.User });
            ClaimsMap.Add("Guest", new List<string> { Constants.Permissions.Read, Constants.Roles.Guest});
        }

        public static void AuthenticateUser(string username, string password)
        {
            string pw;
            if (!Credentials.TryGetValue(username, out pw))
            {
                throw new SecurityException("Access is denied. Username does not exist.");
            }

            if (!pw.Equals(password))
            {
                throw new SecurityException("Access is denied. Invalid password.");
            }
        }

        public static List<Claim> GetPermissionsForUser(string username)
        {
            var result = from item in ClaimsMap[username]
                         where item.Contains(Constants.ClaimTypes.Permission)
                         select new Claim(Constants.ClaimTypes.Permission, item);

            return result as List<Claim>;
        }

        public static List<Claim> GetRolesForUser(string username)
        {
            var result = from item in ClaimsMap[username]
                         where item.Contains(Constants.ClaimTypes.Role)
                         select new Claim(Constants.ClaimTypes.Role, item);

            return result as List<Claim>;
        }

        public static List<Claim> GetClaimsForUser(string username)
        {
            List<Claim> claims = new List<Claim>();

            var result = from item in ClaimsMap[username]
                         where item.Contains(Constants.ClaimTypes.Permission)
                         select new Claim(Constants.ClaimTypes.Permission, item);
            claims.AddRange(result);

            var result2 = from item in ClaimsMap[username]
                         where item.Contains(Constants.ClaimTypes.Role)
                         select new Claim(Constants.ClaimTypes.Role, item);
            claims.AddRange(result2);

            return claims;
        }


        
    }
}
