﻿// © 2009 Michele Leroux Bustamante. All rights reserved. 
// See http://claimsbasedwpf.codeplex.com for related whitepaper and updates
// For an intro to WCF see Michele's book: Learning WCF, O'Reilly 2007 (updated August 2008 for VS 2008)
// See http://www.thatindigogirl.com for the book code!

using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using System.Collections.Generic;
using WpfClient.TodoList;

namespace WpfClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private ObservableCollection<TodoItem> _TodoItems = new ObservableCollection<TodoItem>();
        private string Username { get; set; }
        private string Password { get; set; }

        // to test a regular proxy switch to use TodoListServiceClient
        private TodoListServiceClient _Proxy = new TodoListServiceClient();
                
        /// <summary>
        /// Will hold the newly created Item (top of the screen)
        /// </summary>
        private TodoItem _NewTodo;

        public MainWindow()
        {
            
            InitializeComponent();
            
        }

        void Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            try
            {
                TodoItem item = (TodoItem)sender;
                _Proxy.UpdateItem(item);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

        }

        private void AddTodoButton_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            try
            {
                _NewTodo.CreationDate = DateTime.Today;
                _NewTodo.Description = DescriptionTextBox.Text;
                _NewTodo.DueDate = DueDatePicker.SelectedDate.GetValueOrDefault(DateTime.Today);
                _NewTodo.Title = TitleTextBox.Text;
                _NewTodo.Priority = (PriorityFlag) PriorityComboBox.SelectedItem;

                _NewTodo.ID = _Proxy.CreateItem(_NewTodo);

                _NewTodo.PropertyChanged += Item_PropertyChanged;
                _TodoItems.Add(_NewTodo);
                
                _NewTodo = new TodoItem { Priority = PriorityFlag.Normal, DueDate = DateTime.Today };
                NewItemPanel.DataContext = _NewTodo;

                
                TitleTextBox.Focus();
            }
            catch (Exception ex)
            {
                DescriptionTextBox.Text = _NewTodo.Description;
                TitleTextBox.Text = _NewTodo.Title;
                MessageBox.Show(ex.Message);
            }

        }


        private void DeleteTodoItem_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            try
            {
                TodoItem item = (TodoItem)((Button)sender).DataContext;
                _Proxy.DeleteItem(item.ID);

                item.PropertyChanged -= Item_PropertyChanged;
                _TodoItems.Remove(item);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void window_Closing(object sender, CancelEventArgs e)
        {
            try
            {
                _Proxy.Close();
            }
            catch 
            {
                _Proxy.Abort();
            }
            
        }

        private void window_Loaded(object sender, RoutedEventArgs e)
        {
            _Proxy = new TodoListServiceClient();

            this.NewItemBorder.Visibility = Visibility.Hidden;
            this.TodoDataGrid.Visibility = Visibility.Hidden;

            if (!ShowLogin()) return;

            // initialize the proxy
            this._Proxy.ClientCredentials.UserName.UserName = this.Username;
            this._Proxy.ClientCredentials.UserName.Password = this.Password;
            this._TodoItems = this._Proxy.GetItems();

            foreach (TodoItem item in this._TodoItems)
            {
                item.PropertyChanged += Item_PropertyChanged;
            }

            // Prepare the UI
            PriorityComboBox.ItemsSource = TodoItem.PriorityFlags;
            _NewTodo = new TodoItem { Priority = PriorityFlag.Normal, DueDate = DateTime.Today };
            NewItemPanel.DataContext = _NewTodo;

            this.TodoDataGrid.ItemsSource = this._TodoItems;

            this.NewItemBorder.Visibility = Visibility.Visible;
            this.TodoDataGrid.Visibility = Visibility.Visible;

        }

        private bool ShowLogin()
        {
            LoginForm f = new LoginForm();
            bool? result = f.ShowDialog();

            if (result ==false)
            {
                MessageBox.Show("You must log in before using the application.");
                this.Close();
                return false;
            }

            this.Username = f.Username;
            this.Password = f.Password;
            return true;
        }

   }
}
